--                                                                    --
--  package                         Copyright (c)  Dmitry A. Kazakov  --
--     Generic_Name_Tables                         Luebeck            --
--  Implementation                                 Winter, 2003       --
--                                                                    --
--                                Last revision :  16:12 16 Apr 2005  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.IO_Exceptions;  use Ada.IO_Exceptions;

package body Name_Tables is

   function Check_Matched (Source : String; Pointer : Integer)
      return Boolean is
   begin
      return not Is_In (Source (Pointer), Non_Breaks);
   end Check_Matched;

   procedure Check_Name (Name : String) is
      Symbol : Character;
   begin
      if (  Name'Length = 0
         or else
            not Is_In (Name (Name'First), Capitals)
         )
      then
         raise Constraint_Error;
      end if;
      for Index in Name'First + 1 .. Name'Last loop
         Symbol := Name (Name'First);
         if Is_In (Symbol, Singletons) then
            if Index = Name'Last or else Symbol = Name (Index + 1) then
               raise Constraint_Error;
            end if;
         elsif not Is_In (Symbol, Name_Body) then
            raise Constraint_Error;
         end if;
      end loop;
   end Check_Name;

   function Get_Name (Source : String; Pointer : access Integer)
      return String is
      From   : constant Integer := Pointer.all;
      Index  : Integer          := From;
      Symbol : Character;
   begin
      if Index < Source'First then
         raise Layout_Error;
      end if;
      if Index > Source'Last then
         if Index - 1 > Source'Last then
            raise Layout_Error;
         else
            raise End_Error;
         end if;
      end if;
      if not Is_In (Source (Index), Capitals) then
         raise End_Error;
      end if;
      Index := Index + 1;
      while Index <= Source'Last loop
         Symbol := Source (Index);
         if not Is_In (Symbol, Non_Breaks) then 
            exit when not Is_In (Symbol, Name_Body);
            if Is_In (Symbol, Singletons) then
               exit when Index >= Source'Last;
               Symbol := Source (Index + 1);
               if not Is_In (Symbol, Non_Breaks) then
                  exit when
                     (  not Is_In (Symbol, Name_Body)
                     or else
                        Is_In (Symbol, Singletons)
                     );
               end if;
               Index := Index + 1;
            end if;
         end if;
         Index := Index + 1;
      end loop;
      loop
         exit when Source (Index - 1) /= ' ';
         Index := Index - 1;
      end loop;
      Check_Name (Source (From..Index - 1));
      Pointer.all := Index; 
      return Source (From..Index - 1);
   end Get_Name;

end Name_Tables;
