--                                                                    --
--  package                         Copyright (c)  Dmitry A. Kazakov  --
--     Object.Handle.Bounded_Array                 Luebeck            --
--  Implementation                                 Spring, 2003       --
--                                                                    --
--                                Last revision :  22:07 05 Dec 2004  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

package body Object.Handle.Generic_Bounded_Array is

   procedure Adjust (Container : in out Bounded_Array) is
      Element : Object_Type_Ptr;
   begin
      for Index in Container.Vector'Range loop
         Element := Container.Vector (Index);
         if Element /= null then
            Element.Use_Count := Element.Use_Count + 1;
         end if;
      end loop;
   end Adjust;

   procedure Finalize (Container : in out Bounded_Array) is
   begin
      for Index in Container.Vector'Range loop
         if Container.Vector (Index) /= null then
            Release (Container.Vector (Index));
         end if;
      end loop;
   end Finalize;

   procedure Fill
             (  Container : in out Bounded_Array;
                From      : Index_Type;
                To        : Index_Type;
                Element   : Object_Type_Ptr
             )  is
      subtype Index_Range is Index_Type range Container.Vector'Range;
   begin
      if From > To then
         return;
      else
         if Element = null then
            for Index in Index_Range (From)..Index_Range (To) loop
               Release (Container.Vector (Index));
               Container.Vector (Index) := null;
            end loop;
         else
            declare
               Object : Object_Type'Class renames Element.all;
            begin
               for Index in Index_Range (From)..Index_Range (To) loop
                  if (  Container.Vector (Index) = null
                     or else
                        not Equal
                            (  Container.Vector (Index).all,
                               Object
                     )      )
                  then
                     Release (Container.Vector (Index));
                     Container.Vector (Index) := Element;
                     Object.Use_Count := Object.Use_Count + 1;
                  end if;
               end loop;
            end;
         end if;
      end if;
   end Fill;

   procedure Fill
             (  Container : in out Bounded_Array;
                From      : Index_Type;
                To        : Index_Type;
                Element   : Handle_Type
             )  is
   begin
      Fill (Container, From, To, Ptr (Element));
   end Fill;

   function Get
            (  Container : Bounded_Array;
               Index     : Index_Type
            )  return Object_Type_Ptr is
   begin
      if Index not in Container.Vector'Range then
         return null;
      else
         return Container.Vector (Index);
      end if;
   end Get;

   procedure Put
             (  Container : in out Bounded_Array;
                Index     : Index_Type;
                Element   : Object_Type_Ptr
             )  is
      Item : Object_Type_Ptr renames Container.Vector (Index);
   begin
      if Element = null then
         Release (Item);
         Item := null;
      else
         declare
            Object : Object_Type'Class renames Element.all;
         begin
            if Item /= null and then Equal (Item.all, Object) then
               return;
            end if;
            Release (Item);
            Item := Element;
            Object.Use_Count := Object.Use_Count + 1;
         end;
      end if;
   end Put;

   procedure Put
             (  Container : in out Bounded_Array;
                Index     : Index_Type;
                Element   : Handle_Type
             )  is
   begin
      Put (Container, Index, Ptr (Element));
   end Put;

   function Ref
            (  Container : Bounded_Array;
               Index     : Index_Type
            )  return Handle_Type is
      Ptr : Object_Type_Ptr := Get (Container, Index);
   begin
      if Ptr = null then
         raise Constraint_Error;
      else
         return Ref (Ptr);
      end if;
   end Ref;

end Object.Handle.Generic_Bounded_Array;
