--                                                                    --
--  package Persistent.Handle       Copyright (c)  Dmitry A. Kazakov  --
--  Implementation                                 Luebeck            --
--                                                 Winter, 2002       --
--                                                                    --
--                                Last revision :  12:32 10 Jun 2003  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

package body Persistent.Handle is

   function Get
            (  Storage : Storage_Handle;
               Name    : Wide_String
            )  return Deposit_Handle is
   begin
      return Get (Ptr (Storage), Name);
   end Get;

   function Get_List
            (  Storage     : Storage_Handle;
               Prefix      : Wide_String := "";
               Suffix      : Wide_String := "";
               Equivalence : Wide_Character_Mapping := Identity
            )  return Catalogue.Set is
   begin
      return Get_List (Ptr (Storage), Prefix, Suffix, Equivalence);
   end Get_List;

   function Get_Name
            (  Storage : Storage_Handle;
               Object  : Deposit_Handle
            )  return Wide_String is
   begin
      return Get_Name (Ptr (Storage), Object);
   end Get_Name;

   procedure Invalidate (Storage : in out Storage_Handle) is
   begin
      Handles.Invalidate (Handles.Handle (Storage));
   end Invalidate;
   
   function Is_In
            (  Storage : Storage_Handle;
               Object  : Deposit_Handle
            )  return Boolean is
   begin
      return Is_In (Ptr (Storage), Object);
   end Is_In;

   function Is_In
            (  Storage : Storage_Handle;
               Name    : Wide_String
            )  return Boolean is
   begin
      return Is_In (Ptr (Storage), Name);
   end Is_In;

   function Is_Valid (Storage : Storage_Handle) return Boolean is
   begin
      return Handles.Is_Valid (Handles.Handle (Storage));
   end Is_Valid;

   function Ptr (Storage : Storage_Handle) return Storage_Object_Ptr is
   begin
      return Handles.Ptr (Handles.Handle (Storage));
   end Ptr;

   procedure Put
             (  Storage : in out Storage_Handle;
                Object  : in out Deposit_Handle;
                Name    : Wide_String
             )  is
   begin
      Put (Ptr (Storage).all, Object, Name);
   end Put;

   procedure Put
             (  Storage : in out Storage_Handle;
                Object  : in out Deposit_Handle
             )  is
   begin
      Put (Ptr (Storage).all, Object);
   end Put;

   function Ref (Storage: Storage_Object_Ptr) return Storage_Handle is
   begin
      return (Handles.Ref (Storage) with null record);
   end Ref;

   procedure Rename
             (  Storage  : in out Storage_Handle;
                Object   : Wide_String;
                New_Name : Wide_String
             )  is
   begin
      Rename (Ptr (Storage).all, Object, New_Name);  
   end Rename;

   procedure Rename
             (  Storage  : in out Storage_Handle;
                Object   : in out Deposit_Handle;
                New_Name : Wide_String
             )  is
   begin
      Rename (Ptr (Storage).all, Object, New_Name);
   end Rename;

   procedure Unname
             (  Storage : in out Storage_Handle;
                Object  : Wide_String
             )  is
   begin
      Unname (Ptr (Storage).all, Object);
   end Unname;

   procedure Unname
             (  Storage : in out Storage_Handle;
                Object  : in out Deposit_Handle
             )  is
   begin
      Unname (Ptr (Storage).all, Object);
   end Unname;

end Persistent.Handle;
