--                                                                    --
--  procedure Test_Measures         Copyright (c)  Dmitry A. Kazakov  --
--  Test                                           Luebeck            --
--                                                 Spring, 2000       --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.Calendar;                use type Ada.Calendar.Time;
with Ada.Exceptions;              use Ada.Exceptions;
with Units;                       use Units;
with Units.Base;                  use Units.Base;
with Units.Edit;                  use Units.Edit;
with Units.Constants;             use Units.Constants;
with Float_Measures;              use Float_Measures;
with Float_Measures_Derived;      use Float_Measures_Derived;
with Float_Measures_Irregular;    use Float_Measures_Irregular;
with Text_IO;                     use Text_IO;
with Strings_Edit;                use Strings_Edit;
with Strings_Edit.UTF8.Handling;  use Strings_Edit.UTF8.Handling;

with Float_Measures_Edit;
with Float_Measures_UTF8_Edit;
with Strings_Edit.Floats;

procedure Test_Measures is

   procedure TestUnit (X : Unit; Result : String) is
   begin
      if Image (X) /= Result then
         Put
         (  "Expected:'"
         &  Result
         &  "' Got:'"
         &  Image (X)
         &  "' ('"
         &  Image (X, Latin1 => False)
         &  "')"
         );
         New_Line;
         raise Data_Error;
      end if;
   end TestUnit;

   procedure TestIO
             (  X       : Measure;
                Result  : String;
                Small   : Integer := 0;
                Derived : Boolean := True;
                Latin1  : Boolean := True
             )  is
      use Float_Measures_Edit;
   begin
      if (  (  Result
            /= Image
               (  X,
                  Derived  => Derived,
                  Latin1   => Latin1,
                  AbsSmall => Small
            )  )
         or X /= Value (Result)
         )
      then
         Put
         (  "Expected:'"
         &  Result
         &  "', Got:'"
         &  Image
            (  X,
               Derived  => Derived,
               Latin1   => Latin1,
               AbsSmall => Small
            )
         &  "' (U="
         &  Image (Value (Result).SI, False)
         &  ", G="
         &  Float'Image (Value (Result).Gain)
         &  ", O="
         &  Float'Image (Value (Result).Offset)
         );
         New_Line;
         raise Data_Error;
      end if;
   end TestIO;

   procedure TestUTFIO
             (  X       : Measure;
                Result  : String;
                Small   : Integer := 0;
                Derived : Boolean := True
             )  is
      use Float_Measures_UTF8_Edit;
   begin
      if (  (  Result
            /= Image
               (  X,
                  Derived  => Derived,
                  AbsSmall => Small
            )  )
         or X /= Value (Result)
         )
      then
         Put
         (  "Expected:'"
         &  Result
         &  "', Got:'"
         &  Image
            (  X,
               Derived  => Derived,
               AbsSmall => Small
            )
         &  "' (U="
         &  Image (Value (Result).SI, False)
         &  ", G="
         &  Float'Image (Value (Result).Gain)
         &  ", O="
         &  Float'Image (Value (Result).Offset)
         );
         New_Line;
         raise Data_Error;
      end if;
   end TestUTFIO;

   procedure TestExpression
             (  Test     : String;
                Expected : Measure;
                Next     : Character := ASCII.Nul;
                Restrict : Boolean   := False
             )  is
      use Float_Measures_Edit;
      Result  : Measure;
      Pointer : Integer := Test'First;
   begin
      if Restrict then
         Get_Unit (Test, Pointer, Result);
      else
         Get (Test, Pointer, Result);
      end if;
      if Next = ASCII.Nul then
         if Pointer /= Test'Last + 1 then
            Put
            (  "Error in parsing of '"
            &  Test
            &  "', Unrecognized: '"
            &  Test (Pointer..Test'Last)
            &  "'"
            );
            New_Line;
            raise Data_Error;
         end if;
      else
         if Test (Pointer) /= Next then
            Put
            (  "Error in parsing of '"
            &  Test
            &  "', Rest: '"
            &  Test (Pointer..Test'Last)
            &  "'"
            );
            New_Line;
            raise Data_Error;
         end if;
      end if;
      if Expected /= Result then
         Put
         (  "Expected:'"
         &  Image (Expected, Latin1 => False)
         &  "', Got:'"
         &  Image (Result, Latin1 => False)
         &  "'"
         );
         New_Line;
         raise Data_Error;
      end if;
   end TestExpression;

   procedure TestUTF
             (  Test     : String;
                Expected : Measure;
                Next     : Character := ASCII.Nul
             )  is
      use Float_Measures_UTF8_Edit;
      Result  : Measure;
      Pointer : Integer := Test'First;
   begin
      Get (Test, Pointer, Result);
      if Next = ASCII.Nul then
         if Pointer /= Test'Last + 1 then
            Put
            (  "Error in parsing of '"
            &  Test
            &  "', Unrecognized: '"
            &  Test (Pointer..Test'Last)
            &  "'"
            );
            New_Line;
            raise Data_Error;
         end if;
      else
         if Test (Pointer) /= Next then
            Put
            (  "Error in parsing of '"
            &  Test
            &  "', Rest: '"
            &  Test (Pointer..Test'Last)
            &  "'"
            );
            New_Line;
            raise Data_Error;
         end if;
      end if;
      if Expected /= Result then
         Put
         (  "Expected:'"
         &  Float_Measures_Edit.Image (Expected, Latin1 => False)
         &  "', Got:'"
         &  Float_Measures_Edit.Image (Result, Latin1 => False)
         &  "'"
         );
         New_Line;
         raise Data_Error;
      end if;
   end TestUTF;

   subtype Speed is Measure (Velocity);
   V : Speed := 100.0 * m / h;
begin
   Put_Line ("Testing measures ...");
   
   begin
      TestUnit (Current ** (-4) / Current ** 5, "Current ** (-4) / Current ** 5");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Current ** (-4) * Current ** (-5), "Current ** (-4) * Current ** (-5)");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Current ** (-8) * Current ** (-8), "Current ** (-8) * Current ** (-8)");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Current ** 9, "Current ** 9");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Mass ** 4 * Mass ** 4, "Mass ** 4 * Mass ** 4");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Mass ** 7 * Mass ** 7, "Mass ** 7 * Mass ** 7");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Mass ** 8, "Mass ** 8");
   exception
      when Constraint_Error => null;
   end;
   begin
      TestUnit (Sqrt (Mass), "Sqrt (Mass)");
   exception
      when Constraint_Error => null;
   end;

   TestUnit (Unitless,      "1"           );
   TestUnit (Current,       "A"           );
   TestUnit (Luminescence,  "cd"          );
   TestUnit (Temperature,   "K"           );
   TestUnit (Mass,          "kg"          );
   TestUnit (Length,        "m"           );
   TestUnit (Quantity,      "mol"         );
   TestUnit (Time,          "s"           );
   TestUnit (Area,          "m"          );
   TestUnit (Volume,        "m"          );
   TestUnit (Velocity,      "m/s"         );
   TestUnit (Acceleration,  "m/s"        );
   TestUnit (Force,         "kgm/s"     );
   TestUnit (Pressure,      "kg/ms"     );
   TestUnit (Energy,        "kgm/s"    );
   TestUnit (Power,         "kgm/s"    );
   TestUnit (Charge,        "As"         );
   TestUnit (Potential,     "kgm/As"  );
   TestUnit (Capacitance,   "As^4/kgm");
   TestUnit (Resistance,    "kgm/As" );
   TestUnit (Conductance,   "As/kgm" );
   TestUnit (Inductance,    "kgm/As" );
   TestUnit (Concentration, "mol/m"      );
   TestUnit (Density,       "kg/m"       );
   TestUnit (Luminance,     "cd/m"       );
   TestUnit (Frequency,     "1/s"         );

   TestUnit (Mass / Mass, "1");
   TestUnit (Mass * Mass, "kg");

   TestUnit (Mass ** 0,   "1"     );
      TestUnit (Sqrt (Mass ** 0), "1");
   TestUnit (Mass ** 1,   "kg"    );
   TestUnit (Mass ** 2,   "kg"   );
      TestUnit (Sqrt (Mass ** 2), "kg");
   TestUnit (Mass ** 3,   "kg"   );
   TestUnit (Mass ** 4,   "kg^4"  );
      TestUnit (Sqrt (Mass ** 4), "kg");
   TestUnit (Mass ** 5,   "kg^5"  );
   TestUnit (Mass ** 6,   "kg^6"  );
      TestUnit (Sqrt (Mass ** 6), "kg");
   TestUnit (Mass ** 7,   "kg^7"  );

   TestUnit (Mass **(-1), "1/kg"  );
   TestUnit (Mass **(-2), "1/kg" );
      TestUnit (Sqrt (Mass **(-2)), "1/kg" );
   TestUnit (Mass **(-3), "1/kg" );
   TestUnit (Mass **(-4), "1/kg^4");
      TestUnit (Sqrt (Mass **(-4)), "1/kg" );
   TestUnit (Mass **(-5), "1/kg^5");
   TestUnit (Mass **(-6), "1/kg^6");
      TestUnit (Sqrt (Mass **(-6)), "1/kg");
   TestUnit (Mass **(-7), "1/kg^7");
   TestUnit (Mass **(-8), "1/kg^8");
      TestUnit (Sqrt (Mass **(-8)), "1/kg^4");
   TestExpression ("1 Pa", kg/(m*s**2));
   TestExpression ("Pa", kg/(m*s**2), Restrict => True);
   TestExpression ("Pa/m", kg/(m*s**2), Next => '/', Restrict => True);
   TestExpression ("Pa / m", kg/(m*s**2), Next => ' ', Restrict => True);
   TestExpression (" m/m*s**2]", 1.0 / s**2, ']');
   TestExpression (" m/(m*s**2) and 3 x", 1.0 / s**2 and 3.0, 'x');
   TestExpression (" m - 2*m + m ]", 0.0 * m, ']');
   TestExpression (" m + m - 2*m ", 0.0 * m);
   TestExpression (" 2*m + m ", 3.0 * m);
   TestExpression (" m - (2*m + m) ", -2.0 * m);
   TestExpression (" m - -2*m + -m ", 2.0 * m);
   TestExpression (" 500 * % ", 5.0 * Np);
   TestExpression (" 500*% ", 5.0 * Np);
   TestExpression (" 500 % ", 5.0 * Np);
   TestExpression (" 500% ", 5.0 * Np);
   TestExpression (" 500%]", 5.0 * Np, ']');
   TestExpression (" 500%2", 10.0 * Np);
   TestExpression (" 500% x", 5.0 * Np, 'x');
   TestExpression (" 500%x", 5.0 * Np, 'x');
   TestExpression (" 10**2% x", 1.0 * Np, 'x');
   TestExpression (" 10**(1+1)% x", 1.0 * Np, 'x');
   TestExpression (" 10^(1+1)% x", 1.0 * Np, 'x');
   TestExpression (" 10  **  (  1  +  1  )  %   x", 1.0 * Np, 'x');
   TestExpression (" 10km/s", 10000.0 * m/s);
   TestExpression ("C", K and 273.15);
   TestExpression ("C", K and 273.15, Restrict => True);
   TestExpression ("mC", K/1000.0 and 273.15);
   TestExpression ("mC", K/1000.0 and 273.15, Restrict => True);

   TestIO (Np, "1");
   TestIO (25.0 * Np, "25");
   TestIO (Hz, "Hz");
   TestIO (Ohm, "ohm");
   TestIO (Ohm, "kgm/As", Derived => False);
   TestIO (Ohm, "kg*m^2/A^2*s^3", Derived => False, Latin1 => False);
   TestIO (Hz, "1/s", Derived => False);
   TestIO (25.0 * Hz, "25/s", Derived => False);
   TestIO (Celsius, "K and 273.15", -2);
   TestIO (100.0 * m/s, "100m/s");
   TestIO (m, "m");
   TestIO (h, "3600s");
   TestIO (25.0 * N, "25N");
   TestIO (25.0 * N, "25kgm/s",  Derived => False);
   TestIO (25.0 * N, "25*kg*m/s^2", Derived => False, Latin1 => False);

   TestUTFIO (m*s, To_UTF8 (String'("ms")));
   TestUTFIO (25.0*N, To_UTF8 (String'("25N")));
   TestUTFIO (25.0*N, To_UTF8 (String'("25kgm/s")), Derived => False);

   TestUTF ("1 Pa", kg/(m*s**2));
   TestUTF (" m/m*s**2]", 1.0 / s**2, ']');
   TestUTF (" m/(m*s**2) and 3 x", 1.0 / s**2 and 3.0, 'x');
   TestUTF (" m - 2*m + m ]", 0.0 * m, ']');
   TestUTF (" m + m - 2*m ", 0.0 * m);
   TestUTF (" 2*m + m ", 3.0 * m);
   TestUTF (" m - (2*m + m) ", -2.0 * m);
   TestUTF (" m - -2*m + -m ", 2.0 * m);
   TestUTF (" 500 * % ", 5.0 * Np);
   TestUTF (" 500*% ", 5.0 * Np);
   TestUTF (" 500 % ", 5.0 * Np);
   TestUTF (" 500% ", 5.0 * Np);
   TestUTF (" 500%]", 5.0 * Np, ']');
   TestUTF (" 500%2", 10.0 * Np);
   TestUTF (" 500% x", 5.0 * Np, 'x');
   TestUTF (" 500%x", 5.0 * Np, 'x');
   TestUTF (" 10**2% x", 1.0 * Np, 'x');
   TestUTF (" 10**(1+1)% x", 1.0 * Np, 'x');
   TestUTF (" 10^(1+1)% x", 1.0 * Np, 'x');
   TestUTF (" 10  **  (  1  +  1  )  %   x", 1.0 * Np, 'x');
   TestUTF (" 10km/s", 10000.0 * m/s);
   TestUTF (To_UTF8 (String'("C")),     K and 273.15);
   TestUTF (To_UTF8 (String'("mC")),    K/1000.0 and 273.15);
   TestUTF ("m" & UTF8.Image (16#2103#), K/1000.0 and 273.15);
   TestUTF (UTF8.Image (16#212A#), K);
   TestUTF ("m" & UTF8.Image (16#212A#), K/1000.0);
   TestUTF ("mm", m/1000.0);
   TestUTF ("m" & UTF8.Image (16#212A#), K/1000.0);
   TestUTF ("m" & UTF8.Image (16#3A9#),  Ohm/1000.0);
   TestUTF ("m" & UTF8.Image (16#2126#), Ohm/1000.0);
   TestUTF (UTF8.Image (16#2109#), Fahrenheit);
   TestUTF ("m" & UTF8.Image (16#2070#), Np);
   TestUTF ("m" & UTF8.Image (16#B9#), m);
   TestUTF ("m" & UTF8.Image (16#B2#), m**2);
   TestUTF ("m" & UTF8.Image (16#B3#), m**3);
   TestUTF ("m " & UTF8.Image (16#2075#), m**5);
   TestUTF ("m" & UTF8.Image (16#207A#) & UTF8.Image (16#2075#), m**5);
   TestUTF ("m" & UTF8.Image (16#207A#) & UTF8.Image (16#2076#), m**6);
   TestUTF ("m" & UTF8.Image (16#207A#) & UTF8.Image (16#2077#), m**7);
   TestUTF
   (  "m" & UTF8.Image (16#207B#) & UTF8.Image (16#2075#),
      m**(-5)
   );
   TestUTF
   (  "m" & UTF8.Image (16#207B#) & UTF8.Image (16#2078#),
      m**(-8)
   );

   if 5.0 * Celsius /= (Temperature, 5.0, 273.15) then
      raise Data_Error;
   end if;
   if Celsius + Celsius /= (Temperature, 2.0, 273.15) then
      raise Data_Error;
   end if;
   begin
      TestIO (Celsius * Celsius, "Celsius * Celsius", 0);
   exception
      when Unit_Error => null;
   end;
   begin
      TestIO (Celsius * K, "Celsius * K", 0);
   exception
      when Unit_Error => null;
   end;
   begin
      TestIO (Celsius / K, "Celsius / K", 0);
   exception
      when Unit_Error => null;
   end;
   begin
      TestIO (Celsius + K, "Celsius + K", 0);
   exception
      when Unit_Error => null;
   end;
   begin
      TestIO (Celsius - K, "Celsius - K", 0);
   exception
      when Unit_Error => null;
   end;
   if m * ft /= (Length * Length, 0.3048, 0.0) then
      raise Data_Error;
   end if;
   if m + ft /= (Length, 1.3048, 0.0) then
      raise Data_Error;
   end if;
   --
   -- Performance test
   --
   Put_Line ("Now a small performance test ...");
   declare
      type Revolution  is mod 8;
      type Generator   is array (Revolution) of Float;
      subtype Speed    is Measure (Velocity);
      subtype Distance is Measure (Length);
      subtype TimeSpan is Measure (Time);
      Dice   : Generator  := (0.5, 0.1, 0.2, 0.7, 0.4, 0.6, 0.3, 0.8);
      Count  : Revolution := 0;
      M_Time : Duration;
      F_Time : Duration;
      Sum    : Float;
      Start  : Ada.Calendar.Time;
   begin
      declare
         X : Distance := 0.0 * m;
         V : Speed    := 6.0 * m/s;
         T : TimeSpan := 0.5 * s;
      begin
         Count := 0;
         Start := Ada.Calendar.Clock;
         for Index in 1..3_000_000 loop
            X := X + V * T;
            V := V * Dice (Count); Count := Count + 1;
            T := T * Dice (Count); Count := Count + 1;
         end loop;
         M_Time := Ada.Calendar.Clock - Start;
         Sum := Get_Value (X);
      end;
      declare
         X : Float := 0.0;
         V : Float := 6.0;
         T : Float := 0.5;
      begin
         Count := 0;
         Start := Ada.Calendar.Clock;
         for Index in 1..3_000_000 loop
            X := X + V * T;
            V := V * Dice (Count); Count := Count + 1;
            T := T * Dice (Count); Count := Count + 1;
         end loop;
         F_Time := Ada.Calendar.Clock - Start;
         if Sum /= X then
            raise Unit_Error;
         end if;
      end;
      Put_Line
      (  "Operations with Float : "
      &  Strings_Edit.Floats.Image (Float (F_Time), AbsSmall => -3)
      &  " s"
      );
      Put_Line
      (  "Same with Measure     : "
      &  Strings_Edit.Floats.Image (Float (M_Time), AbsSmall => -3)
      &  " s"
      );
      Put_Line
      (  "Performance hit       : "
      &  Strings_Edit.Floats.Image (Float (M_Time / F_Time), AbsSmall => -2)
      &  " times"
      );
      Put_Line ("... Done");
   end;
exception
   when Error : others =>
      Put ("Error :");
      Put_Line (Exception_Information (Error));
end Test_Measures;
