--                                                                    --
--  procedure                       Copyright (c)  Dmitry A. Kazakov  --
--     Test_Persistent_Tree                        Luebeck            --
--  Interface                                      Autumn, 2004       --
--                                                                    --
--                                Last revision :  22:08 05 Dec 2004  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Object.Archived;  use Object.Archived;
with Deposit_Handles;  use Deposit_Handles;

package Test_Persistent_Tree is
   --
   -- Nothing -- No node handle
   --
   function Nothing return Handle;
   --
   -- Create_Node -- This function creates a new node
   --
   --    Field - Identifies the node
   --    Left  - Successor on the left (a handle to)
   --    Right - Successor on the right (a handle to)
   --
   function Create_Node
            (  Field : Integer;
               Left  : Handle := Nothing;
               Right : Handle := Nothing
            )  return Handle;
   -- 
   -- Print -- Prints the tree rooted in a node
   --
   --   Root - The root node (a handle to)
   --
   procedure Print (Root : Handle; Indentation : String := "");

private
   --
   -- Node -- Binary tree node type
   --
   type Node is new Deposit with record
      Field : Integer; -- Node identifier
      Left  : Handle;  -- Left successor, a handle to
      Right : Handle;  -- Right successor, a handle to
   end record;
   --
   -- Implementation of Deposit's operations
   --
   function Get_Class (Object : Node) return String;
   procedure Get_Referents
             (  Object    : Node;
                Container : in out Deposit_Container'Class
             );
   function Is_Modified (Object : Node) return Boolean;
   procedure Reset_Modified (Object : in out Node);
   procedure Restore
             (  Source  : String;
                Pointer : in out Integer;
                Class   : String;
                List    : Deposit_Container'Class;
                Object  : out Deposit_Ptr
             );
   procedure Store
             (  Destination : in out String;
                Pointer     : in out Integer;
                Object      : Node
             );
end Test_Persistent_Tree;
