--                                                                    --
--  package Unit_Converter          Copyright (c)  Dmitry A. Kazakov  --
--  Sample program for GTK+                        Luebeck            --
--  Implementation                                 Autumn, 2004       --
--                                                                    --
--                                Last revision :  15:50 07 May 2005  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.Characters.Latin_1;    use Ada.Characters.Latin_1;
with Ada.IO_Exceptions;         use Ada.IO_Exceptions;
--
-- Things related to Gtk-API
--
with Gdk.Event;     use Gdk.Event;
with Gtk.Enums;     use Gtk.Enums;
with Gtk.Frame;     use Gtk.Frame;
with Gtk.Button;    use Gtk.Button;
with Gtk.GEntry;    use Gtk.GEntry;
with Gtk.Handlers;  use Gtk.Handlers;
with Gtk.Label;     use Gtk.Label;
with Gtk.Table;     use Gtk.Table;
with Gtk.Widget;    use Gtk.Widget;
with Gtk.Window;    use Gtk.Window;
with Gtk.Main;
--
-- This is what we need for unit conversions
--
with Float_Measures;            use Float_Measures;
with Float_Measures_UTF8_Edit;  use Float_Measures_UTF8_Edit;
with Units;                     use Units;

procedure Units_Converter is

   package Handlers is
      new Gtk.Handlers.Callback (Widget_Type => Gtk_Widget_Record);

   package Return_Handlers is
      new Gtk.Handlers.Return_Callback
          (  Widget_Type => Gtk_Widget_Record,
             Return_Type => Boolean
          );

   function Delete_Event
            (  Widget : access Gtk_Widget_Record'Class;
               Event  : Gdk_Event
            )  return Boolean is
   begin
      return False;
   end Delete_Event;

   procedure Destroy (Widget : access Gtk_Widget_Record'Class) is
   begin
      Gtk.Main.Main_Quit;
   end Destroy;
--
-- Read_Only_Text -- Sunken labels
--
   type Read_Only_Text is record
      Text  : Gtk_Label;
      Frame : Gtk_Frame;
   end record;
--
-- Gtk_New -- Initializes the label
--
   procedure Gtk_New
             (  Text  : in out Read_Only_Text;
                Label : String := ""
             )  is
   begin
      Gtk_New (Text.Frame);
      Set_Shadow_Type (Text.Frame, Shadow_In);
      Gtk_New (Text.Text, Label);
      Set_Alignment (Text.Text, 0.0, 0.5);
      Add (Text.Frame, Text.Text);
   end Gtk_New;
--
-- Show -- The label
--
   procedure Show (Text : in out Read_Only_Text) is
   begin
      Show (Text.Text);
      Show (Text.Frame);
   end Show;

   Window              : Gtk_Window;
   Grid                : Gtk_Table;
   Label1              : Gtk_Label;
   Label2              : Gtk_Label;
   Label3              : Gtk_Label;
   Value_To_Convert    : Gtk_Entry;
   Value_In_SI         : Gtk_Entry;
   Value_In_Base_Units : Gtk_Entry;
   Button              : Gtk_Button;
   Message             : Read_Only_Text;
--
-- Go -- When the Go button gets pressed
--
--    Widget - The button
--
   procedure Go (Widget : access Gtk_Widget_Record'Class) is
      Number : Measure;
   begin
      Set_Text (Value_In_SI, "");
      Set_Text (Value_In_Base_Units, "");
      Set_Text (Message.Text, "");
      Number := Value (Get_Text (Value_To_Convert));
      Set_Text (Value_In_SI, Image (Number));
      Set_Text
      (  Value_In_Base_Units,
         Image (Number, Derived => False)
      );
   exception
      when Constraint_Error =>
         Set_Text (Message.Text, "Numeric error");
      when Data_Error =>
         Set_Text (Message.Text, "Syntax error");
      when End_Error =>
         Set_Text (Message.Text, "Nothing recognized");
      when Unit_Error =>
         Set_Text (Message.Text, "Unit error");
   end Go;

begin
   --
   -- Initialization
   --
   Gtk.Main.Init;
   --
   -- Creating the main window and handle its events
   --
   Gtk.Window.Gtk_New (Window);
   Set_Title (Window, "Unit conversion (Ada95 GTK+)");
   Return_Handlers.Connect
   (  Window,
      "delete_event",
      Return_Handlers.To_Marshaller (Delete_Event'Access)
   );
   Handlers.Connect
   (  Window,
      "destroy",
      Handlers.To_Marshaller (Destroy'Access)
   );
   Gtk.Window.Set_Border_Width (Window, 10);
   --
   -- Creating the grid, a table to align all other widgets
   --
   Gtk_New (Grid, 3, 4, False);
   Set_Row_Spacings (Grid, 3);
   Set_Col_Spacings (Grid, 3);
   Gtk.Window.Add (Window, Grid);
   --
   -- The left column are labels
   --
   Gtk_New (Label1, "Value to convert" & LF & "For example 23.5 bar");
   Attach_Defaults (Grid, Label1, 0, 1, 0, 1);
   Set_Justify (Label1, Justify_Right);
   Set_Alignment (Label1, 1.0, 0.5);
   Gtk_New (Label2, "SI equivalent");
   Attach_Defaults (Grid, Label2, 0, 1, 1, 2);
   Set_Alignment (Label2, 1.0, 0.5);
   Gtk_New (Label3, "Base units only");
   Attach_Defaults (Grid, Label3, 0, 1, 3, 4);
   Set_Alignment (Label3, 1.0, 0.5);
   --
   -- The central column is the edit fields
   --
   Gtk_New (Value_To_Convert);
   Handlers.Connect
   (  Value_To_Convert,
      "activate",
      Handlers.To_Marshaller (Go'Access)
   );
   Attach_Defaults (Grid, Value_To_Convert, 1, 2, 0, 1);
   Gtk_New (Value_In_SI);
   Attach_Defaults (Grid, Value_In_SI, 1, 2, 1, 2);
   Gtk_New (Value_In_Base_Units);
   Attach_Defaults (Grid, Value_In_Base_Units, 1, 2, 3, 4);
   --
   -- The right column is the button Go
   --
   Gtk_New (Button, " Go ");
   Handlers.Connect
   (  Button,
      "clicked",
      Handlers.To_Marshaller (Go'Access)
   );
   Attach_Defaults (Grid, Button, 3, 4, 0, 4);
   --
   -- Error messages is beneath
   --
   Gtk_New (Message, "Source: www.dmitry-kazakov.de/ada/units.htm");
   Attach_Defaults (Grid, Message.Frame, 0, 4, 4, 5);
   --
   -- Show everything
   --
   Show (Label1);  Show (Value_To_Convert);    Show (Button);
   Show (Label2);  Show (Value_In_SI);
   Show (Label3);  Show (Value_In_Base_Units);
   Show (Message);
   
   Show (Grid);
   Show (Window);
   --
   -- Enter the events processing loop
   --
   Gtk.Main.Main;
end Units_Converter;
