--                                                                    --
--  package Units.UTF8_Edit         Copyright (c)  Dmitry A. Kazakov  --
--  Interface                                      Luebeck            --
--                                                 Spring, 2005       --
--                                                                    --
--                                Last revision :  22:14 12 Sep 2005  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

package Units.UTF8_Edit is
--
-- Image -- Unit to string conversion
--
--    Value - To be converted
--
-- This  function is used to convert a unit to string. The syntax of the
-- result is: 
--
--    <result>    ::= <list>/<list> | <list> | 1
--    <list>      ::= <item> [<*><list>]
--    <item>      ::= <base-unit> [<power>] 
--    <base-unit> ::= A | cd | K | kg | m | mol | s
--
-- Here  <*>  is  either  * (if Latin1 is false), or Latin-1 point. When
-- Latin1 is false <power> is always  ^<number>.  Powers  are  indicated
-- using  the  corresponding superscript characters. When all powers are
-- negative then numerator will be one of the negative powers. 
--
-- For instance: 
--
--    m/s -- Velocity unit
--      
-- Returns :
--
--    The string
--
   function Image (Value : Unit) return String;
   
end Units.UTF8_Edit;
